@@ -12,13 +12,14 @@ Trading project:
 
 # TODO:
 #   add inner class with non-trading utils
-#   add stop loss adder
 #   add tradingview realtime signals
+#   numpy
+#   all trading on client
 
 import random
 import time
 import typing
-from typing import Dict, List, Tuple
+from typing import Dict, List, Tuple, Any, Iterable
 
 import numpy as np
 import pandas as pd
@@ -45,7 +46,7 @@ class Trader(object):
     interval: |     str      |  interval of df.
 
     """
-    profit_calculate_coef: float
+    _profit_calculate_coef: float
     returns: utils.PREDICT_TYPE_LIST = []
     df: pd.DataFrame
     ticker: str
@@ -53,106 +54,104 @@ class Trader(object):
     __exit_order__: bool = False
     _old_predict: str = 'Exit'
     _regression_inputs: int
-    stop_loss: float
-    take_profit: float
-    open_price: float
-    history: Dict[str, List[float]]
-    training_set: Tuple[np.ndarray, np.ndarray]
+    _stop_loss: float
+    _take_profit: float
+    _open_price: float
     trades: int = 0
     profits: int = 0
     losses: int = 0
-    stop_losses: List[float]
-    take_profits: List[float]
-    credit_leverages: List[float]
+    _stop_losses: List[float]
+    _take_profits: List[float]
+    _credit_leverages: List[float]
     deposit_history: List[float]
     year_profit: float
-    linear: np.ndarray
+    _linear: np.ndarray
     info: str
-    backtest_out_no_drop: pd.DataFrame
+    _backtest_out_no_drop: pd.DataFrame
     backtest_out: pd.DataFrame
-    open_lot_prices: List[float]
+    _open_lot_prices: List[float]
     realtime_returns: Dict[str, Dict[str, typing.Union[str, float]]]
     client: brokers.TradingClient
     __last_stop_loss: float
     __last_take_profit: float
-    returns_strategy: List[float]
-    sec_interval: int
+    returns_strategy_diff: List[float]
+    _sec_interval: int
+    supports: Dict[int, float]
+    resistances: Dict[int, float]
 
     def __init__(self,
                  ticker: str = 'BTC/USDT',
                  df: pd.DataFrame = pd.DataFrame(),
                  interval: str = '1d',
-                 rounding: int = 50,
                  *args,
                  **kwargs):
-        df_ = round(df, rounding)
-        self.df = df_.reset_index(drop=True)
+        self.df = df.reset_index(drop=True)
         self.ticker = ticker
         self.interval = interval
         if interval == '1m':
-            self.profit_calculate_coef = 1 / (60 * 24 * 365)
-            self.sec_interval = 60
+            self._profit_calculate_coef = 1 / (60 * 24 * 365)
+            self._sec_interval = 60
         elif interval == '2m':
-            self.profit_calculate_coef = 1 / (30 * 24 * 365)
-            self.sec_interval = 120
+            self._profit_calculate_coef = 1 / (30 * 24 * 365)
+            self._sec_interval = 120
         elif interval == '3m':
-            self.profit_calculate_coef = 1 / (20 * 24 * 365)
-            self.sec_interval = 180
+            self._profit_calculate_coef = 1 / (20 * 24 * 365)
+            self._sec_interval = 180
         elif interval == '5m':
-            self.profit_calculate_coef = 1 / (12 * 24 * 365)
-            self.sec_interval = 300
+            self._profit_calculate_coef = 1 / (12 * 24 * 365)
+            self._sec_interval = 300
         elif interval == '15m':
-            self.profit_calculate_coef = 1 / (4 * 24 * 365)
-            self.sec_interval = 15 * 60
+            self._profit_calculate_coef = 1 / (4 * 24 * 365)
+            self._sec_interval = 15 * 60
         elif interval == '30m':
-            self.profit_calculate_coef = 1 / (2 * 24 * 365)
-            self.sec_interval = 60 * 30
+            self._profit_calculate_coef = 1 / (2 * 24 * 365)
+            self._sec_interval = 60 * 30
         elif interval == '45m':
-            self.profit_calculate_coef = 1 / (32 * 365)
-            self.sec_interval = 60 * 45
+            self._profit_calculate_coef = 1 / (32 * 365)
+            self._sec_interval = 60 * 45
         elif interval == '1h':
-            self.profit_calculate_coef = 1 / (24 * 365)
-            self.sec_interval = 60 * 60
+            self._profit_calculate_coef = 1 / (24 * 365)
+            self._sec_interval = 60 * 60
         elif interval == '90m':
-            self.profit_calculate_coef = 1 / (18 * 365)
-            self.sec_interval = 60 * 90
+            self._profit_calculate_coef = 1 / (18 * 365)
+            self._sec_interval = 60 * 90
         elif interval == '2h':
-            self.profit_calculate_coef = 1 / (12 * 365)
-            self.sec_interval = 60 * 60 * 2
+            self._profit_calculate_coef = 1 / (12 * 365)
+            self._sec_interval = 60 * 60 * 2
         elif interval == '3h':
-            self.profit_calculate_coef = 1 / (8 * 365)
-            self.sec_interval = 60 * 60 * 3
+            self._profit_calculate_coef = 1 / (8 * 365)
+            self._sec_interval = 60 * 60 * 3
         elif interval == '4h':
-            self.profit_calculate_coef = 1 / (6 * 365)
-            self.sec_interval = 60 * 60 * 4
+            self._profit_calculate_coef = 1 / (6 * 365)
+            self._sec_interval = 60 * 60 * 4
         elif interval == '12h':
-            self.profit_calculate_coef = 1 / (2 * 365)
-            self.sec_interval = 60 * 60 * 12
+            self._profit_calculate_coef = 1 / (2 * 365)
+            self._sec_interval = 60 * 60 * 12
         elif interval == '1d':
-            self.profit_calculate_coef = 1 / 365
-            self.sec_interval = 60 * 60 * 24
+            self._profit_calculate_coef = 1 / 365
+            self._sec_interval = 60 * 60 * 24
         elif interval == '3d':
-            self.profit_calculate_coef = 1 / (365 / 3)
-            self.sec_interval = 86400 * 3
+            self._profit_calculate_coef = 1 / (365 / 3)
+            self._sec_interval = 86400 * 3
         elif interval == '1w':
-            self.profit_calculate_coef = 1 / 52
-            self.sec_interval = 86400 * 7
+            self._profit_calculate_coef = 1 / 52
+            self._sec_interval = 86400 * 7
         elif interval == '1M':
-            self.profit_calculate_coef = 1 / 12
-            self.sec_interval = 86400 * 30
+            self._profit_calculate_coef = 1 / 12
+            self._sec_interval = 86400 * 30
         elif interval == '3M':
-            self.profit_calculate_coef = 1 / 4
-            self.sec_interval = 86400 * 90
+            self._profit_calculate_coef = 1 / 4
+            self._sec_interval = 86400 * 90
         elif interval == '6M':
-            self.profit_calculate_coef = 1 / 2
-            self.sec_interval = 86400 * 180
+            self._profit_calculate_coef = 1 / 2
+            self._sec_interval = 86400 * 180
         else:
-            raise ValueError(f'I N C O R R E C T   I N T E R V A L; {interval}')
+            raise ValueError(f'incorrect interval; {interval}')
         self._regression_inputs = utils.REGRESSION_INPUTS
         self.__exit_order__ = False
 
     def __repr__(self):
-        return 'trader'
+        return f'{self.ticker} {self.interval} trader'
 
     def _get_attr(self, attr: str):
         return getattr(self, attr)
@@ -225,38 +224,69 @@ class Trader(object):
 
         _stop_loss: float
         take: float
-        if self.stop_loss is not np.inf:
-            _stop_loss = self.stop_loss / 10_000 * self.open_price
+        if self._stop_loss is not np.inf:
+            _stop_loss = self._stop_loss / 10_000 * self._open_price
         else:
             _stop_loss = np.inf
-        if self.take_profit is not np.inf:
-            take = self.take_profit / 10_000 * self.open_price
+        if self._take_profit is not np.inf:
+            take = self._take_profit / 10_000 * self._open_price
         else:
             take = np.inf
 
         if sig == utils.BUY:
-            _stop_loss = self.open_price - _stop_loss
-            take = self.open_price + take
+            _stop_loss = self._open_price - _stop_loss
+            take = self._open_price + take
         elif sig == utils.SELL:
-            take = self.open_price - take
-            _stop_loss = self.open_price + _stop_loss
+            take = self._open_price - take
+            _stop_loss = self._open_price + _stop_loss
         else:
-            if self.take_profit is not np.inf:
-                take = self.open_price
-            if self.stop_loss is not np.inf:
-                _stop_loss = self.open_price
+            if self._take_profit is not np.inf:
+                take = self._open_price
+            if self._stop_loss is not np.inf:
+                _stop_loss = self._open_price
         utils.logger.debug(
-            f'stop loss: {_stop_loss} ({self.stop_loss} pips), take profit: {take} ({self.take_profit} pips)')
+            f'stop loss: {_stop_loss} ({self._stop_loss} pips), take profit: {take} ({self._take_profit} pips)')
 
         return {'stop': _stop_loss,
                 'take': take}
 
+    def sl_tp_adder(self, add_stop_loss: float = 0, add_take_profit: float = 0) -> Tuple[List[float], List[float]]:
+        """
+
+        :param add_stop_loss: add stop loss points
+        :param add_take_profit: add take profit points
+        :return: (stop losses, take profits)
+        """
+        stop_losses = []
+        take_profits = []
+        for stop_loss_price, take_profit_price, price, sig in zip(self._stop_losses,
+                                                                  self._take_profits,
+                                                                  self.df['Close'].values,
+                                                                  self.returns):
+            add_sl = (price / 10_000) * add_stop_loss
+            add_tp = (price / 10_000) * add_take_profit
+
+            if sig == utils.BUY:
+                stop_losses.append(stop_loss_price - add_sl)
+                take_profits.append(take_profit_price + add_tp)
+            elif sig == utils.SELL:
+                stop_losses.append(stop_loss_price + add_sl)
+                take_profits.append(take_profit_price - add_tp)
+            else:
+                stop_losses.append(stop_loss_price)
+                take_profits.append(take_profit_price)
+
+        self._stop_losses = stop_losses
+        self._take_profits = take_profits
+        return self._stop_losses, self._take_profits
+
     def strategy_diff(self, frame_to_diff: pd.Series, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
         """
         frame_to_diff:  |   pd.Series  |  example:  Trader.df['Close']
         """
         self.returns = list(np.digitize(frame_to_diff.diff(), bins=[0]))
         self.set_open_stop_and_take()
+        self.set_credit_leverages()
         return self.returns
 
     def strategy_buy_hold(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
@@ -425,7 +455,7 @@ class Trader(object):
                                                              self.df['Close'], **sar_kwargs)
         sardown: np.ndarray = sar.psar_down().values
         sarup: np.ndarray = sar.psar_up().values
-        self.stop_losses = list(sar.psar().values)
+        self._stop_losses = list(sar.psar().values)
 
         if plot:
             for SAR_ in (sarup, sardown):
@@ -483,9 +513,9 @@ class Trader(object):
             self.fig.add_trace(Line(y=st.get_supertrend_lower(),
                                     name='supertrend lower',
                                     line=dict(width=utils.SUB_LINES_WIDTH, color=utils.GREEN)))
-        self.stop_losses = list(st.get_supertrend())
+        self._stop_losses = list(st.get_supertrend())
         self.returns = list(st.get_supertrend_strategy_returns())
-        self.stop_losses[0] = np.inf if self.returns[0] == utils.SELL else -np.inf
+        self._stop_losses[0] = np.inf if self.returns[0] == utils.SELL else -np.inf
         self.set_open_stop_and_take(set_stop=False)
         return self.returns
 
@@ -610,7 +640,7 @@ class Trader(object):
                 line_color=utils.ICHIMOKU_CLOUD_COLOR))
 
             self.returns = [utils.EXIT for i in range(chinkouspan)]
-            self.stop_losses = [np.inf] * chinkouspan
+            self._stop_losses = [np.inf] * chinkouspan
             for e, (close, tenkan, kijun, A, B) in enumerate(zip(
                     prices.values[chinkouspan:],
                     tenkan_sen[chinkouspan:],
@@ -645,14 +675,27 @@ class Trader(object):
                         trade = utils.EXIT
                 self.returns.append(trade)
                 if trade == utils.BUY:
-                    self.stop_losses.append(min_cloud - stop_loss_adder)
+                    self._stop_losses.append(min_cloud - stop_loss_adder)
                 else:
-                    self.stop_losses.append(max_cloud + stop_loss_adder)
+                    self._stop_losses.append(max_cloud + stop_loss_adder)
         self.set_open_stop_and_take(set_take=True,
                                     set_stop=False)
         self.set_open_stop_and_take()
         return self.returns
 
+    def crossover(self, fast: Iterable, slow: Iterable):
+        self.returns = []
+        for s, f in zip(slow, fast):
+            if s < f:
+                self.returns.append(utils.BUY)
+            elif s > f:
+                self.returns.append(utils.SELL)
+            else:
+                self.returns.append(utils.EXIT)
+        self.set_credit_leverages()
+        self.set_open_stop_and_take()
+        return self.returns
+
     def inverse_strategy(self, swap_tpop_take: bool = True, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
         """
         makes signals inverse:
@@ -673,7 +716,7 @@ class Trader(object):
             returns.append(flag)
         self.returns = returns
         if swap_tpop_take:
-            self.stop_losses, self.take_profits = self.take_profits, self.stop_losses
+            self._stop_losses, self._take_profits = self._take_profits, self._stop_losses
         return self.returns
 
     def backtest(self,
@@ -702,14 +745,17 @@ class Trader(object):
         no_order: bool
         stop_loss: float
         take_profit: float
-        seted: List[typing.Any]
+        seted: List[Any]
         diff: float
         lin_calc_df: pd.DataFrame
-        price: float
+        high: float
+        low: float
         credit_lev: float
 
         start_bet: float = bet
         data_column: pd.Series = self.df[column]
+        data_high: pd.Series = self.df['High']
+        data_low: pd.Series = self.df['Low']
         self.deposit_history = [deposit]
         seted_ = utils.set_(self.returns)
         self.trades = 0
@@ -722,16 +768,24 @@ class Trader(object):
 
         e: int
         sig: utils.PREDICT_TYPE
+        ignore_breakout: bool = False
         for e, (sig,
                 stop_loss,
                 take_profit,
                 seted,
-                credit_lev) in enumerate(zip(self.returns[:-1],
-                                             self.stop_losses[:-1],
-                                             self.take_profits[:-1],
-                                             seted_[:-1],
-                                             self.credit_leverages[:-1]), 1):
-            price = data_column[e]
+                credit_lev,
+                high,
+                low,
+                next_h,
+                next_l) in enumerate(zip(self.returns[:-1],
+                                         self._stop_losses[:-1],
+                                         self._take_profits[:-1],
+                                      seted_[:-1],
+                                         self._credit_leverages[:-1],
+                                      data_high[:-1],
+                                      data_low[:-1],
+                                      data_high[1:],
+                                      data_low[1:])):
 
             if seted is not np.nan:
                 if oldsig != utils.EXIT:
@@ -740,34 +794,31 @@ class Trader(object):
                     commission = start_commission
                 if bet > deposit:
                     bet = deposit
-                open_price = price
+                open_price = data_column[e]
                 deposit -= bet * (commission / 100) * credit_lev
                 if bet > deposit:
                     bet = deposit
-                self.trades += 1
-                if deposit > moneys_open_bet:
-                    self.profits += 1
-                elif deposit < moneys_open_bet:
-                    self.losses += 1
                 moneys_open_bet = deposit
                 no_order = False
                 exit_take_stop = False
+                ignore_breakout = True
 
-            if min(stop_loss, take_profit) < price < max(stop_loss, take_profit):
-                diff = data_column[e] - data_column[e - 1]
+            next_breakout = min(stop_loss, take_profit) < next_l <= next_h < max(stop_loss, take_profit)
+            if (min(stop_loss, take_profit) < low <= high < max(stop_loss, take_profit) and next_breakout) or ignore_breakout:
+                diff = data_column[e + 1] - data_column[e]
             else:
                 exit_take_stop = True
-                if sig == utils.BUY and price >= take_profit:
-                    diff = take_profit - data_column[e - 1]
+                if sig == utils.BUY and high >= take_profit:
+                    diff = take_profit - data_column[e]
 
-                elif sig == utils.BUY and price <= stop_loss:
-                    diff = stop_loss - data_column[e - 1]
+                elif sig == utils.BUY and low <= stop_loss:
+                    diff = stop_loss - data_column[e]
 
-                elif sig == utils.SELL and price >= stop_loss:
-                    diff = stop_loss - data_column[e - 1]
+                elif sig == utils.SELL and high >= stop_loss:
+                    diff = stop_loss - data_column[e]
 
-                elif sig == utils.SELL and price <= take_profit:
-                    diff = take_profit - data_column[e - 1]
+                elif sig == utils.SELL and low <= take_profit:
+                    diff = take_profit - data_column[e]
 
                 else:
                     diff = 0.0
@@ -781,37 +832,49 @@ class Trader(object):
             no_order = exit_take_stop
             self.deposit_history.append(deposit)
             oldsig = sig
+            if seted is not np.nan:
+                if sig != utils.EXIT:
+                    self.trades += 1
+                    if deposit > moneys_open_bet:
+                        self.profits += 1
+                    elif deposit < moneys_open_bet:
+                        self.losses += 1
+            ignore_breakout = False
 
-        self.linear = utils.get_linear(self.deposit_history)
-        lin_calc_df = pd.DataFrame(self.linear)
+        self._linear = utils.get_linear(self.deposit_history)
+        lin_calc_df = pd.DataFrame(self._linear)
         mean_diff = float(lin_calc_df.diff().mean())
-        self.year_profit = mean_diff / self.profit_calculate_coef
+        self.year_profit = mean_diff / self._profit_calculate_coef
         self.year_profit = (self.year_profit / money_start) * 100
-        self.winrate = (self.profits / self.trades) * 100
-        self.info = f"""losses: {self.losses}
+        if self.trades != 0:
+            self.winrate = (self.profits / self.trades) * 100
+        else:
+            self.winrate = 0
+        self._info = f"""losses: {self.losses}
 trades: {self.trades}
 profits: {self.profits}
 mean year percentage profit: {self.year_profit}%
 winrate: {self.winrate}%"""
-        utils.logger.info(f'trader info: {self.info}')
+        utils.logger.info(f'trader info: {self._info}')
         if print_out:
-            print(self.info)
-        self.returns_strategy = list(pd.Series(self.deposit_history).diff().values)
-        self.backtest_out_no_drop = pd.DataFrame(
-            (self.deposit_history, self.stop_losses, self.take_profits, self.returns,
-             self.open_lot_prices, data_column, self.linear, self.returns_strategy),
+            print(self._info)
+        self.returns_strategy_diff = list(pd.Series(self.deposit_history).diff().values)
+        self.returns_strategy_diff[0] = 0
+        self._backtest_out_no_drop = pd.DataFrame(
+            (self.deposit_history, self._stop_losses, self._take_profits, self.returns,
+             self._open_lot_prices, data_column, self._linear, self.returns_strategy_diff),
             index=[
                 f'deposit ({column})', 'stop loss', 'take profit',
                 'predictions', 'open deal/lot', column,
                 f"linear deposit data ({column})",
                 "returns"
             ]).T
-        self.backtest_out = self.backtest_out_no_drop.dropna()
+        self.backtest_out = self._backtest_out_no_drop.dropna()
         if plot:
             loc: pd.Series = self.df[column]
             self.fig.add_trace(
                 Line(
-                    y=self.backtest_out_no_drop['returns'].values,
+                    y=self._backtest_out_no_drop['returns'].values,
                     line=dict(color=utils.COLOR_DEPOSIT),
                     name='returns'
                 ),
@@ -828,7 +891,7 @@ winrate: {self.winrate}%"""
                 name=f'{self.ticker} {self.interval}')
             self.fig.add_trace(
                 Line(
-                    y=self.take_profits,
+                    y=self._take_profits,
                     line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.GREEN),
                     opacity=utils.STOP_TAKE_OPN_ALPHA,
                     name='take profit'),
@@ -836,7 +899,7 @@ winrate: {self.winrate}%"""
                 col=1)
             self.fig.add_trace(
                 Line(
-                    y=self.stop_losses,
+                    y=self._stop_losses,
                     line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.RED),
                     opacity=utils.STOP_TAKE_OPN_ALPHA,
                     name='stop loss'),
@@ -844,7 +907,7 @@ winrate: {self.winrate}%"""
                 col=1)
             self.fig.add_trace(
                 Line(
-                    y=self.open_lot_prices,
+                    y=self._open_lot_prices,
                     line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.BLUE),
                     opacity=utils.STOP_TAKE_OPN_ALPHA,
                     name='open lot'),
@@ -854,15 +917,15 @@ winrate: {self.winrate}%"""
                 Line(
                     y=self.deposit_history,
                     line=dict(color=utils.COLOR_DEPOSIT),
-                    name=f'D E P O S I T  (S T A R T: ${money_start})'), 2, 1)
-            self.fig.add_trace(Line(y=self.linear, name='L I N E A R'), 2, 1)
+                    name=f'deposit (start: ${money_start})'), 2, 1)
+            self.fig.add_trace(Line(y=self._linear, name='linear'), 2, 1)
             preds: Dict[str, List[typing.Union[int, float]]] = {'sellind': [],
                                                                 'exitind': [],
                                                                 'buyind': [],
                                                                 'bprice': [],
                                                                 'sprice': [],
                                                                 'eprice': []}
-            for e, i in enumerate(utils.set_(self.returns)):
+            for e, i in enumerate(seted_, 0):
                 if i == utils.SELL:
                     preds['sellind'].append(e)
                     preds['sprice'].append(loc[e])
@@ -899,13 +962,100 @@ winrate: {self.winrate}%"""
 
         return self.backtest_out
 
+    def multi_backtest(self,
+                       tickers: List[str],
+                       strategy_name: str = 'strategy_macd',
+                       strategy_kwargs: Dict[str, Any] = {},
+                       deposit: float = 10_000.0,
+                       bet: float = np.inf,
+                       commission: float = 0.0,
+                       plot: bool = True,
+                       print_out: bool = True,
+                       column: str = 'Close',
+                       show: bool = True,
+                       limit: int = 1000) -> pd.DataFrame:
+        winrates: List[float] = []
+        percentage_profits: List[float] = []
+        losses: List[int] = []
+        trades: List[int] = []
+        profits: List[int] = []
+        depo: List[List[float]] = []
+
+        for ticker in tickers:
+            df = self.client.get_data_historical(ticker=ticker, limit=limit, interval=self.interval)
+            new_trader = self._get_this_instance(interval=self.interval, df=df, ticker=ticker)
+            new_trader.set_client(your_client=self.client)
+            new_trader.set_pyplot()
+            new_trader._get_attr(strategy_name)(**strategy_kwargs)
+            new_trader.backtest(deposit=deposit / len(tickers),
+                                bet=bet,
+                                commission=commission,
+                                plot=False,
+                                print_out=False,
+                                column=column,
+                                show=False)
+            winrates.append(new_trader.winrate)
+            percentage_profits.append(new_trader.year_profit)
+            losses.append(new_trader.losses)
+            trades.append(new_trader.trades)
+            profits.append(new_trader.profits)
+            depo.append(new_trader.deposit_history)
+        self.losses = sum(losses)
+        self.trades = sum(profits)
+        self.profits = sum(profits)
+        self.year_profit = float(np.mean(percentage_profits))
+        self.winrate = float(np.mean(winrates))
+        self.deposit_history = list(sum(np.array(depo)))
+        self._linear = utils.get_linear(self.deposit_history)
+        self.returns_strategy_diff = list(pd.Series(self.deposit_history).diff().values)
+        self.returns_strategy_diff[0] = 0
+        self._backtest_out_no_drop = pd.DataFrame(
+            (self.deposit_history, self._linear, self.returns_strategy_diff),
+            index=[
+                f'deposit ({column})',
+                f"linear deposit data ({column})",
+                "returns"
+            ]).T
+        self.backtest_out = self._backtest_out_no_drop.dropna()
+
+        self._info = f"""losses: {self.losses}
+trades: {self.trades}
+profits: {self.profits}
+mean year percentage profit: {self.year_profit}%
+winrate: {self.winrate}%"""
+        utils.logger.info(f'trader multi info: {self._info}')
+        if print_out:
+            print(self._info)
+        if plot:
+            self.fig.add_trace(
+                Line(
+                    y=self.deposit_history,
+                    line=dict(color=utils.COLOR_DEPOSIT),
+                    name=f'deposit (start: ${deposit})'), 2, 1)
+            self.fig.add_trace(Line(y=self._linear, name='linear'), 2, 1)
+            self.fig.add_trace(
+                Line(
+                    y=self.returns_strategy_diff,
+                    line=dict(color=utils.COLOR_DEPOSIT),
+                    name='returns'
+                ),
+                row=3,
+                col=1
+            )
+        if show:
+            self.fig.show()
+        return self.backtest_out
+
     def set_pyplot(self,
                    height: int = 900,
                    width: int = 1300,
                    template: str = 'plotly_dark',
                    row_heights: list = [10, 16, 7],
+                   quick_trade_logo: bool = True,
                    **subplot_kwargs):
         """
+
+        :param quick_trade_logo: use quick trade logo as background
         :param height: window height
         :param width: window width
         :param template: plotly template
@@ -1008,7 +1158,7 @@ winrate: {self.winrate}%"""
         elif mode == 'super':
             self.returns = self.__collide_super(first_returns, second_returns)
         else:
-            raise ValueError('I N C O R R E C T   M O D E')
+            raise ValueError('incorrect mode')
         return self.returns
 
     @staticmethod
@@ -1059,7 +1209,7 @@ winrate: {self.winrate}%"""
         :return: dict with prediction
         """
 
-        credit_leverage: float = self.credit_leverages[-1]
+        credit_leverage: float = self._credit_leverages[-1]
         _moneys_: float
         bet: float
         close: np.ndarray = self.df["Close"].values
@@ -1074,11 +1224,11 @@ winrate: {self.winrate}%"""
             self.__exit_order__ = False
 
         # trading
-        self.__last_stop_loss = self.stop_losses[-1]
-        self.__last_take_profit = self.take_profits[-1]
+        self.__last_stop_loss = self._stop_losses[-1]
+        self.__last_take_profit = self._take_profits[-1]
         if self._old_predict != predict:
             utils.logger.info(f'open lot {predict}')
-            self.open_price = close[-1]
+            self._open_price = close[-1]
             if trading_on_client:
 
                 if predict == 'Exit':
@@ -1103,7 +1253,7 @@ winrate: {self.winrate}%"""
                     self.__exit_order__ = False
         return {
             'predict': predict,
-            'open lot price': self.open_price,
+            'open lot price': self._open_price,
             'stop loss': self.__last_stop_loss,
             'take profit': self.__last_take_profit,
             'currency close': close[-1]
@@ -1119,9 +1269,11 @@ winrate: {self.winrate}%"""
                          ignore_exceptions: bool = True,
                          print_exc: bool = True,
                          wait_sl_tp_checking: float = 5,
+                         limit: int = 1000,
                          *strategy_args,
                          **strategy_kwargs):
         """
+        :param limit: client.get_data_historical's limit argument
         :param wait_sl_tp_checking: sleeping time after stop-loaa and take-profit checking (seconds)
         :param print_exc: print  exceptions in while loop
         :param ignore_exceptions: ignore binance exceptions in while loop
@@ -1137,57 +1289,52 @@ winrate: {self.winrate}%"""
 
         self.realtime_returns = {}
         self.ticker = ticker
-        try:
-            __now__ = time.time()
-            while True:
-                try:
-                    self.df = self.client.get_data_historical(ticker=self.ticker, limit=1000, interval=self.interval)
-                    strategy(*strategy_args, **strategy_kwargs)
+        __now__ = time.time()
+        while True:
+            try:
+                self.df = self.client.get_data_historical(ticker=self.ticker, limit=limit, interval=self.interval)
+                strategy(*strategy_args, **strategy_kwargs)
 
-                    prediction = self.get_trading_predict(
-                        trading_on_client=trading_on_client,
-                        bet_for_trading_on_client=bet_for_trading_on_client,
-                        coin_lotsize_division=coin_lotsize_division)
+                prediction = self.get_trading_predict(
+                    trading_on_client=trading_on_client,
+                    bet_for_trading_on_client=bet_for_trading_on_client,
+                    coin_lotsize_division=coin_lotsize_division)
 
-                    index = f'{self.ticker}, {time.ctime()}'
-                    utils.logger.info(f"trading prediction at {index}: {prediction}")
-                    if print_out:
-                        print(index, prediction)
-                    self.realtime_returns[index] = prediction
-                    while True:
-                        if not self.__exit_order__:
-                            price = self.client.get_ticker_price(ticker)
-                            min_ = min(self.__last_stop_loss, self.__last_take_profit)
-                            max_ = max(self.__last_stop_loss, self.__last_take_profit)
-                            if (not (min_ < price < max_)) and prediction["predict"] != 'Exit':
-                                self.__exit_order__ = True
-                                utils.logger.info('exit lot')
-                                prediction['predict'] = 'Exit'
-                                prediction['currency close'] = price
-                                index = f'{self.ticker}, {time.ctime()}'
-                                utils.logger.info(f"trading prediction exit in sleeping at {index}: {prediction}")
-                                if print_out:
-                                    print(f"trading prediction exit in sleeping at {index}: {prediction}")
-                                self.realtime_returns[index] = prediction
-                                if trading_on_client:
-                                    self.client.exit_last_order()
-                        time.sleep(wait_sl_tp_checking)
-                        if not (time.time() < (__now__ + self.sec_interval)):
-                            self._old_predict = utils.convert_signal_str(self.returns[-1])
-                            __now__ += self.sec_interval
-                            break
-                except Exception as exc:
-                    if ignore_exceptions:
-                        if print_exc:
-                            print(exc)
-                        utils.logger.error('error :(', exc_info=True)
-                        continue
-                    else:
-                        raise exc
-
-        except Exception as e:
-            utils.logger.critical('error :(', exc_info=True)
-            raise e
+                index = f'{self.ticker}, {time.ctime()}'
+                utils.logger.info(f"trading prediction at {index}: {prediction}")
+                if print_out:
+                    print(index, prediction)
+                self.realtime_returns[index] = prediction
+                while True:
+                    if not self.__exit_order__:
+                        price = self.client.get_ticker_price(ticker)
+                        min_ = min(self.__last_stop_loss, self.__last_take_profit)
+                        max_ = max(self.__last_stop_loss, self.__last_take_profit)
+                        if (not (min_ < price < max_)) and prediction["predict"] != 'Exit':
+                            self.__exit_order__ = True
+                            utils.logger.info('exit lot')
+                            prediction['predict'] = 'Exit'
+                            prediction['currency close'] = price
+                            index = f'{self.ticker}, {time.ctime()}'
+                            utils.logger.info(f"trading prediction exit in sleeping at {index}: {prediction}")
+                            if print_out:
+                                print(f"trading prediction exit in sleeping at {index}: {prediction}")
+                            self.realtime_returns[index] = prediction
+                            if trading_on_client:
+                                self.client.exit_last_order()
+                    time.sleep(wait_sl_tp_checking)
+                    if not (time.time() < (__now__ + self._sec_interval)):
+                        self._old_predict = utils.convert_signal_str(self.returns[-1])
+                        __now__ += self._sec_interval
+                        break
+            except Exception as exc:
+                if ignore_exceptions:
+                    if print_exc:
+                        print(exc)
+                    utils.logger.error('error :(', exc_info=True)
+                    continue
+                else:
+                    raise exc
 
     def log_data(self, *args, **kwargs):
         self.fig.update_yaxes(row=1, col=1, type='log')
@@ -1234,15 +1381,15 @@ winrate: {self.winrate}%"""
         :param take_profit: take profit in points
         :param stop_loss: stop loss in points
         """
-        self.take_profit = take_profit
-        self.stop_loss = stop_loss
+        self._take_profit = take_profit
+        self._stop_loss = stop_loss
         take_flag: float = np.inf
         stop_flag: float = np.inf
-        self.open_lot_prices = []
+        self._open_lot_prices = []
         if set_stop:
-            self.stop_losses = []
+            self._stop_losses = []
         if set_take:
-            self.take_profits = []
+            self._take_profits = []
         closes: np.ndarray = self.df['Close'].values
         sig: utils.PREDICT_TYPE
         close: float
@@ -1250,18 +1397,18 @@ winrate: {self.winrate}%"""
         ts: Dict[str, float]
         for sig, close, seted in zip(self.returns, closes, utils.set_(self.returns)):
             if seted is not np.nan:
-                self.open_price = close
+                self._open_price = close
                 if set_take or set_stop:
                     ts = self.__get_stop_take(sig)
                 if set_take:
                     take_flag = ts['take']
                 if set_stop:
                     stop_flag = ts['stop']
-            self.open_lot_prices.append(self.open_price)
+            self._open_lot_prices.append(self._open_price)
             if set_take:
-                self.take_profits.append(take_flag)
+                self._take_profits.append(take_flag)
             if set_stop:
-                self.stop_losses.append(stop_flag)
+                self._stop_losses.append(stop_flag)
         utils.logger.debug(f'trader stop loss: {stop_loss}, trader take profit: {take_profit}')
 
     def set_credit_leverages(self, credit_lev: float = 1.0, *args, **kwargs):
@@ -1269,14 +1416,14 @@ winrate: {self.winrate}%"""
         Sets the leverage for bets.
         :param credit_lev: leverage in points
         """
-        self.credit_leverages = [credit_lev for i in range(len(self.df['Close']))]
+        self._credit_leverages = [credit_lev for i in range(len(self.df['Close']))]
         utils.logger.debug(f'trader credit leverage: {credit_lev}')
 
     def _window_(self,
                  column: str,
                  n: int = 2,
                  *args,
-                 **kwargs) -> List[typing.Any]:
+                 **kwargs) -> List[Any]:
         return utils.get_window(self.df[column].values, n)
 
     def find_pip_bar(self,
@@ -1317,7 +1464,7 @@ winrate: {self.winrate}%"""
         flag: utils.PREDICT_TYPE = utils.EXIT
 
         flag_stop_loss: float = np.inf
-        self.stop_losses = [flag_stop_loss]
+        self._stop_losses = [flag_stop_loss]
         high: List[float]
         low: List[float]
         open_pr: List[float]
@@ -1337,7 +1484,7 @@ winrate: {self.winrate}%"""
                 flag_stop_loss = max(high[0], high[1])
 
             self.returns.append(flag)
-            self.stop_losses.append(flag_stop_loss)
+            self._stop_losses.append(flag_stop_loss)
         self.set_open_stop_and_take(set_take=False, set_stop=False)
         return self.returns
 
@@ -1472,3 +1619,14 @@ winrate: {self.winrate}%"""
         patterns = map(utils.anti_set_, patterns)
         self.set_open_stop_and_take()
         return self.multi_strategy_collider(*patterns, mode='super')
+
+    def get_support_resistanse(self) -> Dict[str, Dict[int, float]]:
+        lows = self.df['Low'].values
+        highs = self.df['High'].values
+        for i in range(2, len(lows)-2):
+            if lows[i-2] >= lows[i-1] >= lows[i] <= lows[i+1] <= lows[i+2]:
+                self.supports[i] = lows[i]
+            if highs[i-2] <= highs[i-1] <= highs[i] >= highs[i+1] >= highs[i+2]:
+                self.resistances[i] = highs[i]
+        return {'resistance': self.resistances,
+                'supports': self.supports}
